/*
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:      lo1.c

   Purpose:   To demonstrate how to solve a small linear
              optimization problem using the MOSEK API.
               
 */
  
#include <stdio.h>

#include "mosek.h" /* Include the MOSEK definition file. */

#define NUMCON 3   /* Number of constraints.             */
#define NUMVAR 4   /* Number of variables.               */
#define NUMANZ 9   /* Number of non-zeros in A.          */

static void MSKAPI printstr(void *handle,
                            char str[])
{
  printf("%s",str);
} /* printstr */
  
int main(int argc,char *argv[])
{
  MSKrescodee  r;
  MSKidxt      i,j;
  double       c[]    = {3.0, 1.0, 5.0, 1.0};
  MSKlidxt     ptrb[] = {0, 2, 5, 7};
  MSKlidxt     ptre[] = {2, 5, 7, 9};

  MSKidxt      asub[] = { 0, 1,
                          0, 1, 2,
                          0, 1,
                          1, 2};
  
  double       aval[] = { 3.0, 2.0,
                          1.0, 1.0, 2.0,
                          2.0, 3.0,
                          1.0, 3.0};
 
  MSKboundkeye bkc[]  = {MSK_BK_FX, MSK_BK_LO,     MSK_BK_UP    };
  double       blc[]  = {30.0,      15.0,          -MSK_INFINITY};
  double       buc[]  = {30.0,      +MSK_INFINITY, 25.0         };
  
  MSKboundkeye bkx[]  = {MSK_BK_LO,     MSK_BK_RA, MSK_BK_LO,     MSK_BK_LO     };
  double       blx[]  = {0.0,           0.0,       0.0,           0.0           };
  double       bux[]  = {+MSK_INFINITY, 10.0,      +MSK_INFINITY, +MSK_INFINITY };
  double       xx[NUMVAR];
               
  MSKenv_t     env;
  MSKtask_t    task;

  /* Create the mosek environment. */
  r = MSK_makeenv(&env,NULL,NULL,NULL,NULL);

  /* Check if return code is ok. */
  if ( r==MSK_RES_OK )
  {
    /* Direct the environment log stream to 
       the 'printstr' function. */
    MSK_linkfunctoenvstream(env,MSK_STREAM_LOG,NULL,printstr);
  }

  /* Initialize the environment. */
  r = MSK_initenv(env);

  if ( r==MSK_RES_OK )
  {
    /* Send a message to the MOSEK Message stream. */
    MSK_echoenv(env,
                MSK_STREAM_MSG,
                "Making the MOSEK optimization task\n");

    /* Create the optimization task. */
    r = MSK_maketask(env,NUMCON,NUMVAR,&task);

    if ( r==MSK_RES_OK )
    {
      /* Direct the log task stream to 
         the 'printstr' function. */
      MSK_linkfunctotaskstream(task,MSK_STREAM_LOG,NULL,printstr);
      
      r = MSK_inputdata(task,
                        NUMCON,NUMVAR,
                        NUMCON,NUMVAR,
                        c,
                        0.0,
                        ptrb,
                        ptre,
                        asub,
                        aval,
                        bkc,
                        blc,
                        buc,
                        bkx,
                        blx,
                        bux);   
      
      if ( r==MSK_RES_OK )
      {
        MSK_putobjsense(task,MSK_OBJECTIVE_SENSE_MAXIMIZE);

        MSK_echotask(task,
                     MSK_STREAM_MSG,
                     "Start optimizing\n");

        r = MSK_optimize(task);

        if ( r==MSK_RES_OK )
        {
          MSK_getsolutionslice(task,
                               MSK_SOL_BAS,    /* Request the basic solution. */
                               MSK_SOL_ITEM_XX,/* Which part of solution.     */
                               0,              /* Index of first variable.    */
                               NUMVAR,         /* Index of last variable+1.   */
                               xx);
          printf("Primal solution\n");
          for(j=0; j<NUMVAR; ++j)
            printf("x[%d]: %e\n",j,xx[j]);
        }
      }
    }
 
    MSK_deletetask(&task);
  }
  MSK_deleteenv(&env);

  printf("Return code: %d (0 means no error occured.)\n",r);

  return ( r );
} /* main */
